/*
								+----------------------------------+
								|                                  |
								| ***   Multi-platform timer   *** |
								|                                  |
								|   Copyright  -tHE SWINe- 2006   |
								|                                  |
								|             Timer.h              |
								|                                  |
								+----------------------------------+
*/

/*
 *	passed code revision at 2006-08-23
 *
 *	removed some unnecessary #defines for linux
 *	added higher precision timer for linux (instead of clock(), gettimeofday() can be used)
 *
 *	2007-05-10
 *
 *	added conditional #include <windows.h> to make it easier to include
 *
 *	2008-04-09
 *
 *	added GetTickCount method, redesigned timer code. should cope nicely with counter overflow
 *	(previous version of timer with QueryPerformanceCounter had some trouble on core duo cpu's)
 *
 */

#ifndef __TIMER_INCLUDED
#define __TIMER_INCLUDED

/*
 *	- this macro enables use of QueryPerformanceCounter() function (win32-only)
 */
#define TIMER_ALLOW_QPC

/*
 *	- this macro enables use of GetTickCount() function (win32-only)
 */
#define TIMER_ALLOW_GETTICKCOUNT

/*
 *	- this macro enables use of gettimeofday() function (unix-only)
 */
#define TIMER_ALLOW_GETTIMEOFDAY

/*
 *	- this macro disables automatic timer method selection
 *	- one of following macros must be defined as well to choose timer method:
 *		TIMER_USE_QPC				(win32-only, resolution less than 1 usec, depends on cpu)
 *		TIMER_USE_GETTICKCOUNT		(win32-only, resolution 1 msec)
 *		TIMER_ALLOW_GETTIMEOFDAY	(unix-only, resolution 1 usec)
 *		TIMER_USE_CLOCK				(default fallback, resolution 1 msec or less, depends on os)
 */
//#define TIMER_METHOD_OVERRIDE

#ifndef TIMER_METHOD_OVERRIDE
#ifdef WIN32
#if defined(TIMER_ALLOW_QPC)
#define TIMER_USE_QPC
#elif defined(TIMER_ALLOW_GETTICKCOUNT)
#define TIMER_USE_GETTICKCOUNT
#else
#define TIMER_USE_CLOCK
#endif
#else //WIN32
#ifdef TIMER_ALLOW_GETTIMEOFDAY
#define TIMER_USE_GETTIMEOFDAY
#else
#define TIMER_USE_CLOCK
#endif
#endif //WIN32
#endif //TIMER_METHOD_OVERRIDE

#include "Integer.h"

class CTimer {
	int64_t m_n_freq, m_n_time;
	double m_f_time;

public:
	/*
	 *	CTimer::CTimer()
	 *		- default constructor
	 */
	CTimer();

	/*
	 *	void CTimer::ResetTimer()
	 *		- resets timer (sets time to zero)
	 */
	void ResetTimer();

	/*
	 *	double CTimer::f_Time()
	 *		- returns time in seconds
	 *		- should cope nicely with counter overflows
	 */
	double f_Time();

	/*
	 *	int64_t CTimer::n_Frequency() const
	 *		- returns timer frequency (inverse of smallest time step)
	 */
	int64_t n_Frequency() const;

protected:
	static inline int64_t n_SampleTimer();
};

#endif //__TIMER_INCLUDED
